#!/bin/bash

################################################################
# The Dwarf Fortress Mac OS X App ReBundler ShellScript! v0.07 #
################################################################


#### For convenience: Internal Field Seperator set to Tab only
IFS=$'\t'


#### Global error log
LOGFILE="$(echo $0 | sed 's!.sh$!!g').log"
date > "$LOGFILE"
ERRORFLAG="#ERROR#"

#### Some functions

## echo to stderr
function errorecho()
{
   echo "$@" | tee -a "$LOGFILE" >&2
}

## error to logfile
function errorlogger()
{
   "$@" 2>> "$LOGFILE" || echo "$ERRORFLAG" >> "$LOGFILE"
}

## machine info to logfile
function machineinfo()
{
   INFOS=$(system_profiler | grep Machine | sed 's/^ *//g')
   errorecho "Logging following machine infos:"
   errorecho "$INFOS"
   errorecho ""
   echo $INFOS >> "$LOGFILE" 
}

## Error flag handler
function errorflag()
{
   NUMBERERR=$(cat "$LOGFILE" | grep $ERRORFLAG | wc -l | sed 's! !!g')

   if [ "$NUMBERERR" -gt 0 ]; then
      errorecho "WARNING: I logged $NUMBERERR errors! Please post $LOGFILE to the DF forum thread."
   fi

   echo "$NUMBERERR"
}

## Error/exit on non-empty argument list
function errorexit()
{
   if [ $# -gt 0 ]; then
      if [ $# == 1 ]; then
         errorecho "I fear I have to stop here..." 
      else
         errorecho "I found $# reasons to stop here..." 
      fi
      errorecho "Please post $LOGFILE to the DF forum thread."
      exit 1
   fi
}

## Warn on non-empty argument list
function warngoon()
{
   if [ $# -gt 0 ]; then
      if [ $# == 1 ]; then
         errorecho "There might be a problem. I'll try to go on..."
      else
         errorecho "There might be $# problems. I'll try to go on..."
      fi
      errorecho ""
   fi
}

## Check whether there are commandline arguments, there should be none
function argumenthandler()
{
   if [ $# -gt 0 ]; then
      errorecho "You specified $# arguments to this script." 
      errorecho "If this was intended, you are probably using it wrong."
      errorecho "If this was by accident, please retry without the arguments."
      exit 1
   fi
}

## Drop a warning and wait some time
function warnwait()
{
   errorecho "$2"
   sleep "$1"
}

## Clean DS_Store and Thumbs.db files
function cleanupmess()
{
   errorecho "Cleanup: Mac OS X .DS_Store files"
   find . -name '.DS_Store' -type f -delete
   errorecho "Cleanup: Windows Thumbs.db files"
   find . -name 'Thumbs.db' -type f -delete
}

## Check list of programs, report those missing
function whicharemissing()
{
   errorecho "Let's check necessary programs:"
   MISSLIST=""

   for PROG in "$@"; do
      if [ $(which "$PROG") ]; then
         errorecho "- $PROG found."
      else
         errorecho "- $PROG NOT found."
         MISSLIST="$PROG$IFS$MISSLIST"
      fi
   done
   
   if [ -n "$MISSLIST" ]; then
      errorecho "Missing: $MISSLIST"
   else
      errorecho "Found all programs."
   fi

   errorecho ""
   echo "$MISSLIST"
}

## Check list of directory, report those missing
function whichdonotexist()
{
   errorecho "Let's check game directories:"
   MISSLIST=""

   for DIR in "$@"; do
      if [ -d "$DIR" ]; then
         errorecho "- $DIR exists."
      else
         errorecho "- $DIR does NOT exist."
         MISSLIST="$DIR$IFS$MISSLIST"
      fi
   done
   
   if [ -n "$MISSLIST" ]; then
      errorecho "Missing: $MISSLIST"
   else
      errorecho "Found all directories."
   fi

   errorecho ""
   echo "$MISSLIST"
}

## PlistBuddyBuddy - Try to find PlistBuddy for later usage
function PBB()
{
   PBNAME="PlistBuddy"
   PLISTBUDDY="$(which $1)"
   errorecho "Let's try to find $PBNAME:"

   ## Check whether PlistBuddy is in $PATH
   if [ ! -n "$PLISTBUDDY" ]; then
      if [ $(which "$PBNAME") ]; then
         PLISTBUDDY="$(which $PBNAME)"
         errorecho "- $PBNAME is in your PATH"
      else
         errorecho "- $PBNAME is NOT in your PATH."
      fi
   fi
   
   ## Try /usr/libexec/
   if [ ! -n "$PLISTBUDDY" ]; then
      USRLIBEXEC="/usr/libexec/"
      if [ -x "$USRLIBEXEC$PBNAME" ]; then
         PLISTBUDDY="$USRLIBEXEC$PBNAME"
         errorecho "- $PBNAME is in $USRLIBEXEC."
      else
         errorecho "- $PBNAME is NOT in $USRLIBEXEC."
      fi
   fi
   
   ## Locate PlistBuddy
   if [ ! -n "$PLISTBUDDY" ]; then
      if [ -n "$(errorlogger locate "/$PBNAME")" ]; then
         PLISTBUDDY=$(locate "/$PBNAME" | sed 2,1000000d)
         errorecho "- $PBNAME located."
      else
         errorecho "- $PBNAME NOT located."
      fi
   fi
   
   ## If nothing located: Fallback to direct search in receipts
   if [ ! -n "$PLISTBUDDY" ]; then
      if [ -n "$(errorlogger find /Library/Receipts -name "$PBNAME")" ]; then
         PLISTBUDDY=$(errorlogger find /Library/Receipts -name "$PBNAME" | sed 2,1000000d)
         errorecho "- $PBNAME found."
      else
         errorecho "- $PBNAME NOT found."
      fi
   fi
   
   ## If nothing found: Use 'true' as dummy, and do nothing (successfully) later on
   if [ ! -n "$PLISTBUDDY" ]; then
         errorecho "- Ultima Ratio: Will use dummy."
         PLISTBUDDY="$(which true)"
   fi

   ## Make sure whatever was set as PlistBuddy is executable
   if [ -x "$PLISTBUDDY" ]; then
      errorecho "Will use $PLISTBUDDY as $PBNAME."
      echo "$PLISTBUDDY"
   else
      errorecho "$PLISTBUDDY is not exectuable, will use dummy."
      echo "echo The following command will NOT work: $PBNAME"
   fi

   errorecho ""
}

## Version lookup
function LookupVersion()
{
   cat "$WORKINGDIR/$DIRTEMPLATE/$DIRDOCS/release notes.txt" | grep "Release notes for" | cut -d\  -f4 | cut -d: -f1 | sed 2,1000000d
}

## Year lookup
function LookupYear()
{
   cat "$WORKINGDIR/$DIRTEMPLATE/$DIRDOCS/release notes.txt" | grep "Release notes for" | cut -d\  -f7 | cut -d\) -f1 | sed 2,1000000d
}


#### Define some constants

## Get Working dir
WORKINGDIR="$(pwd)"

## Some constants
NAMEAPP="Dwarf Fortress.app"
NAMEEXEC="Dwarf Fortress"
NEWNAME="newDF"
VERSION="0"
YEAR="0"

DIRTEMPLATE="Dwarf Fortress 0.28.181.40d"
DIRDF="df_osx"

DIRDOCS="docs"
DIRDATA="data"
DIRRAW="raw"
DIRTRICK="workaround"
DIRCONT="Contents"
DIREXEC="$NAMEAPP/$DIRCONT/MacOS"
DIRRES="$NAMEAPP/$DIRCONT/Resources"
DIRFW="$NAMEAPP/$DIRCONT/Frameworks"

FULLDIRDOCS="$WORKINGDIR/$DIRTEMPLATE/$DIRDOCS"
FULLEXEC="$WORKINGDIR/$DIRTEMPLATE/$DIREXEC/$NAMEEXEC"

EXTDOC="txt"


## Find PlistBuddy for later usage
PLISTBUDDY=$(PBB)


#### Sanity Checks

## Handle the arguments
argumenthandler "$@"

## Are the programs there? Probably unnecessary
warngoon $(whicharemissing cat chmod cp cut echo find grep locate ls mkdir mv rm sed touch true wc which system_profiler) #unset exit

## Are the directories there?
errorexit $(whichdonotexist "$DIRTEMPLATE" "$DIRDF")

## Log machine infos from system-profiler
machineinfo


#### Main script

## Here we go!
errorecho "Seems good so far, so let's give this a try..."

## Last chance to stop!
warnwait "5" "...but I'll give you one last chance (15 seconds) to stop here!"
warnwait "5" "- 10 seconds to go. You can cancel a script with CTRL+c, you know?"
warnwait "2" "- 5 seconds to go. "
warnwait "1" "- Three!"
warnwait "1" "- Two!"
warnwait "1" "- One!"
errorecho "Here we go!"
errorecho ""

## Remove directory from last time
if [ -d "$NEWNAME" ]; then
   errorecho "- Remove old $NEWNAME. Your new .app will be in there in a second..."
   rm -rf "$NEWNAME"
fi

## Clean up textfiles
errorecho "- Gather old and new textfiles in $FULLDIRDOCS/."
errorlogger mkdir -p "$FULLDIRDOCS"
errorlogger mv "$WORKINGDIR/$DIRTEMPLATE/"*.$EXTDOC "$FULLDIRDOCS"
errorlogger mv "$WORKINGDIR/$DIRDF/"*.$EXTDOC "$FULLDIRDOCS"
mv "$WORKINGDIR/$DIRDF/sdl/"*.$EXTDOC "$FULLDIRDOCS" 2>/dev/null
mv "$WORKINGDIR/$DIRDF/SDL/"*.$EXTDOC "$FULLDIRDOCS" 2>/dev/null

## Remove old data/raw
errorecho "- Remove old data and raw files."
errorlogger rm -rf "$WORKINGDIR/$DIRTEMPLATE/$DIRDATA"
errorlogger rm -rf "$WORKINGDIR/$DIRTEMPLATE/$DIRRAW"

## Remove old executable
errorecho "- Remove old executables."
errorlogger rm "$WORKINGDIR/$DIRTEMPLATE/$DIREXEC/"* 

## Fix version in Info.plist (That's what PlistBuddy is for...!)
VERSION=$(LookupVersion)
errorecho "- Change version string to $VERSION in Info.plist with PlistBuddy."
"$PLISTBUDDY" "$WORKINGDIR/$DIRTEMPLATE/$NAMEAPP/$DIRCONT/Info.plist" -c "set CFBundleVersion $VERSION"

## Fix About box in InfoPlist.strings. Add about.txt ... Hope that satisfies licensing needs as mentioned in 0.31.13 changelog.
YEAR=$(LookupYear)
errorecho "- Create new InfoPlist.strings for corrected About box informations."
echo "/* About box informations */" > "$WORKINGDIR/$DIRTEMPLATE/$DIRRES/English.lproj/InfoPlist.strings"
echo "NSHumanReadableCopyright = \"Copyright (C) 2002-$YEAR Tarn Adams\nSee docs/about.txt for further information.\";" >> "$WORKINGDIR/$DIRTEMPLATE/$DIRRES/English.lproj/InfoPlist.strings"
errorecho "- Create about.txt for additional About box informations."
echo "Copyright (C) 2002-$YEAR Tarn Adams, Bay 12 Games" > "$FULLDIRDOCS"/about.txt
echo "" >> "$FULLDIRDOCS"/about.txt
echo "This version of DF uses the FMOD Sound System" >> "$FULLDIRDOCS"/about.txt
echo "Copyright (C) 1994-2006 Firelight Technologies Pty, Ltd." >> "$FULLDIRDOCS"/about.txt
echo "" >> "$FULLDIRDOCS"/about.txt
echo "This version of DF uses Simple DirectMedia Layer (SDL) and SDL_Image" >> "$FULLDIRDOCS"/about.txt
echo "Copyright (C) 1997-2006 Sam Lantinga" >> "$FULLDIRDOCS"/about.txt
echo "See the docs folder for license information." >> "$FULLDIRDOCS"/about.txt
echo "" >> "$FULLDIRDOCS"/about.txt
echo "Portions of this version are" >> "$FULLDIRDOCS"/about.txt
echo "copyright (C) 2008 The FreeType Project (www.freetype.org)." >> "$FULLDIRDOCS"/about.txt
echo "All rights reserved." >> "$FULLDIRDOCS"/about.txt

## Create inner .app (workaround for data/raw folder position)
errorecho "- Create inner .app to work around data/raw folder location."
errorlogger mkdir -p "$WORKINGDIR/$DIRTEMPLATE/$DIRRES/$DIRTRICK"
errorlogger mkdir -p "$WORKINGDIR/$DIRTEMPLATE/temp/"
errorlogger cp -r "$WORKINGDIR/$DIRTEMPLATE/$NAMEAPP" "$WORKINGDIR/$DIRTEMPLATE/temp/$NAMEAPP"
errorlogger mv "$WORKINGDIR/$DIRTEMPLATE/temp/$NAMEAPP" "$WORKINGDIR/$DIRTEMPLATE/$DIRRES/$DIRTRICK"
errorlogger rm -rf "$WORKINGDIR/$DIRTEMPLATE/temp/"

## Create outer startup-shellscript
errorecho "- Create outer startup-shellscript."
errorlogger touch "$FULLEXEC"
echo '#!/bin/bash' >> "$FULLEXEC"
echo 'cd "$(dirname "$0")"' >> "$FULLEXEC"
echo "cd ../Resources/$DIRTRICK/" >> "$FULLEXEC"
echo "./$DIREXEC/$NAMEEXEC" | sed 's/\ /\\\ /g' >> "$FULLEXEC"
chmod +x "$FULLEXEC"

## Move data and libs in place
errorecho "- Move data, libs and frameworks in place."
errorlogger mv "$WORKINGDIR/$DIRDF/data" "$WORKINGDIR/$DIRTEMPLATE/$DIRRES/$DIRTRICK"
errorlogger mv "$WORKINGDIR/$DIRDF/raw" "$WORKINGDIR/$DIRTEMPLATE/$DIRRES/$DIRTRICK"
errorlogger mkdir -p "$WORKINGDIR/$DIRTEMPLATE/$DIRRES/$DIRTRICK/$DIRFW"
errorlogger mv "$WORKINGDIR/$DIRDF/libs/"*.framework "$WORKINGDIR/$DIRTEMPLATE/$DIRRES/$DIRTRICK/$DIRFW"/
errorlogger mv "$WORKINGDIR/$DIRDF/libs" "$WORKINGDIR/$DIRTEMPLATE/$DIRRES/$DIRTRICK/$DIREXEC"
errorlogger mv "$WORKINGDIR/$DIRDF/dwarfort.exe" "$WORKINGDIR/$DIRTEMPLATE/$DIRRES/$DIRTRICK/$DIREXEC/$NAMEEXEC"

## Remove remaining old files
errorecho "- Remove remaining old files."
errorlogger rm -rf "$WORKINGDIR/$DIRDF"

## Rename old folder, since it is new now!
errorecho "- Rename $DIRTEMPLATE to $NEWNAME."
errorlogger mv "$WORKINGDIR/$DIRTEMPLATE" "$WORKINGDIR/$NEWNAME"

## Say bye...
if [ ! "$(errorflag)" -gt 0 ]; then
   errorecho "All done!"
   errorecho ""
   errorecho "Have a look in $NEWNAME - bye!"
fi


#### Reset Internal Field Seperator and exit
unset IFS
exit 0


